/**
 * @author Rémi Heredero
 * @version 1.0.0
 * @date July 2023
 * @file button.h
 */
#ifndef BUTTON_H
#define BUTTON_H

#include "../xf/xf.h"

typedef enum {
    STPB_INIT,
    STPB_RELEASED,
    STPB_PRESSED
} BUTTON_STATES;

typedef enum {
    evPBinit = 50,
    evPBpoll
} BUTTON_EVENTS;

typedef void (*BUTTON_CALLBACK_FUNCTION)(void*);
typedef struct {
    BUTTON_CALLBACK_FUNCTION f; // function
    void* p; // param(s)
} BUTTON_CALLBACK;

typedef struct {
    BUTTON_STATES state;
    uint8_t id;
    uint8_t pollTime;
    BUTTON_CALLBACK released;
    BUTTON_CALLBACK pressed;
} BUTTON;

/**
 * Initialize the BUTTON
 * @param me the BUTTON itself
 * @param id the id of the BUTTON
 */
void BUTTON_init(BUTTON* me, uint8_t id);

/**
 * Initialize the hardware of the BUTTON
 * @param me the BUTTON itself
 */
void BUTTON_initHW(BUTTON* me);

/**
 * @brief Check if the button is pressed
 * The function returns true if the button is pressed, false otherwise
 *
 * @param me button itself
 * @return true if the button is pressed
 * @return false if the button is not pressed
 */
bool BUTTON_isPressed(BUTTON* me);

/**
 * Start the BUTTON state machine
 * @param me the BUTTON itself
 */
void BUTTON_startBehaviour(BUTTON* me);

/**
 * Process the event
 * @param ev the event to process
 * @return true if the event is processed
 */
bool BUTTON_processEvent(Event* ev);

/*************
 * Callbacks *
 *************/

/**
 * Set the callback function to call when the BUTTON is entering state released
 * @param me the BUTTON itself
 * @param f the function to call
 * @param p the param(s) to pass to the function
 */
void BUTTON_onReleased(BUTTON* me, BUTTON_CALLBACK_FUNCTION f, void* p);

/**
 * Set the callback function to call when the BUTTON is entering state pressed
 * @param me the BUTTON itself
 * @param f the function to call
 * @param p the param(s) to pass to the function
 */
void BUTTON_onPressed(BUTTON* me, BUTTON_CALLBACK_FUNCTION f, void* p);

/************
 * EMITTERS *
 ************/

/**
 * Emit the poll event
 * @param me the BUTTON itself
 * @param t time to wait in ms before triggering event
 */void BUTTON_emitPoll(BUTTON* me, uint16_t t);

/***********
 * SETTERS *
 ***********/

/**
 * Set the id of the BUTTON
 * @param me the BUTTON itself
 * @param v the id to set
 */
void BUTTON_setId(BUTTON* me, uint8_t v);

/**
 * Set the poll time of the BUTTON
 * @param me the BUTTON itself
 * @param v the poll time to set
 */
void BUTTON_setPollTime(BUTTON* me, uint8_t v);

#endif
