/**
 * @author Rémi Heredero
 * @version 1.0.0
 * @date July 2023
 * @file click_handler.h
 */
#ifndef CLICK_HANDLER_H
#define CLICK_HANDLER_H

#include "../xf/xf.h"

typedef enum {
    STCH_INIT,
    STCH_WAIT,
    STCH_LONG_CLICK,
    STCH_SINGLE_CLICK,
    STCH_DOUBLE_CLICK
} CLICK_HANDLER_STATES;

typedef enum {
    evCHinit = 100,
    evCHtimer,
    evCHpbpress,
    evCHpbrelease
} CLICK_HANDLER_EVENTS;

typedef void (*CLICK_HANDLER_CALLBACK_FUNCTION)(void*);
typedef struct {
    CLICK_HANDLER_CALLBACK_FUNCTION f; // function
    void* p; // param(s)
} CLICK_HANDLER_CALLBACK;

typedef struct {
    CLICK_HANDLER_STATES state;
    uint16_t clickTimer;
    CLICK_HANDLER_CALLBACK wait;
    CLICK_HANDLER_CALLBACK long_click;
    CLICK_HANDLER_CALLBACK single_click;
    CLICK_HANDLER_CALLBACK double_click;
} CLICK_HANDLER;

/**
 * Initialize the CLICK_HANDLER
 * @param me the CLICK_HANDLER itself
 */
void CLICK_HANDLER_init(CLICK_HANDLER* me);

/**
 * Start the CLICK_HANDLER state machine
 * @param me the CLICK_HANDLER itself
 */
void CLICK_HANDLER_startBehaviour(CLICK_HANDLER* me);

/**
 * Process the event
 * @param ev the event to process
 * @return true if the event is processed
 */
bool CLICK_HANDLER_processEvent(Event* ev);

/*************
 * Callbacks *
 *************/

/**
 * Set the callback function to call when the CLICK_HANDLER is entering state wait
 * @param me the CLICK_HANDLER itself
 * @param f the function to call
 * @param p the param(s) to pass to the function
 */
void CLICK_HANDLER_onWait(CLICK_HANDLER* me, CLICK_HANDLER_CALLBACK_FUNCTION f, void* p);

/**
 * Set the callback function to call when the CLICK_HANDLER is entering state long_click
 * @param me the CLICK_HANDLER itself
 * @param f the function to call
 * @param p the param(s) to pass to the function
 */
void CLICK_HANDLER_onLong_click(CLICK_HANDLER* me, CLICK_HANDLER_CALLBACK_FUNCTION f, void* p);

/**
 * Set the callback function to call when the CLICK_HANDLER is entering state single_click
 * @param me the CLICK_HANDLER itself
 * @param f the function to call
 * @param p the param(s) to pass to the function
 */
void CLICK_HANDLER_onSingle_click(CLICK_HANDLER* me, CLICK_HANDLER_CALLBACK_FUNCTION f, void* p);

/**
 * Set the callback function to call when the CLICK_HANDLER is entering state double_click
 * @param me the CLICK_HANDLER itself
 * @param f the function to call
 * @param p the param(s) to pass to the function
 */
void CLICK_HANDLER_onDouble_click(CLICK_HANDLER* me, CLICK_HANDLER_CALLBACK_FUNCTION f, void* p);

/************
 * EMITTERS *
 ************/

/**
 * Emit the timer event
 * @param me the CLICK_HANDLER itself
 * @param t time to wait in ms before triggering event
 */void CLICK_HANDLER_emitTimer(CLICK_HANDLER* me, uint16_t t);

/**
 * Emit the pbpress event
 * @param me the CLICK_HANDLER itself
 * @param t time to wait in ms before triggering event
 */void CLICK_HANDLER_emitPbpress(CLICK_HANDLER* me, uint16_t t);

/**
 * Emit the pbrelease event
 * @param me the CLICK_HANDLER itself
 * @param t time to wait in ms before triggering event
 */void CLICK_HANDLER_emitPbrelease(CLICK_HANDLER* me, uint16_t t);

/***********
 * SETTERS *
 ***********/

void CLICK_HANDLER_setClickTimer(CLICK_HANDLER* me, uint16_t v);

#endif
